@@ -3,7 +3,7 @@ use compute::statistics::*;
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 
 pub fn criterion_mean(c: &mut Criterion) {
-    let v = Normal::new(0., 100.).sample_vec(1e6 as usize);
+    let v = Normal::new(0., 100.).sample_n(1e6 as usize);
     c.bench_function("mean 1e6", |b| b.iter(|| mean(black_box(&v))));
     c.bench_function("welford mean 1e6", |b| {
         b.iter(|| welford_mean(black_box(&v)))
