@@ -35,6 +35,7 @@ impl Default for Bernoulli {
 }
 
 impl Distribution for Bernoulli {
+    type Output = f64;
     /// Samples from the given Bernoulli distribution.
     fn sample(&self) -> f64 {
         if self.p == 1. {
@@ -43,12 +44,15 @@ impl Distribution for Bernoulli {
             return 0.;
         }
 
-        if self.p > fastrand::f64() {
+        if self.p > alea::f64() {
             1.
         } else {
             0.
         }
     }
+}
+
+impl Distribution1D for Bernoulli {
     fn update(&mut self, params: &[f64]) {
         self.set_p(params[0]);
     }
@@ -71,6 +75,7 @@ impl Discrete for Bernoulli {
 }
 
 impl Mean for Bernoulli {
+    type MeanType = f64;
     /// Calculates the mean of the Bernoulli distribution, which is `p`.
     fn mean(&self) -> f64 {
         self.p
@@ -78,6 +83,7 @@ impl Mean for Bernoulli {
 }
 
 impl Variance for Bernoulli {
+    type VarianceType = f64;
     /// Calculates the variance, given by `p*q = p(1-p)`.
     fn var(&self) -> f64 {
         self.p * (1. - self.p)
@@ -92,7 +98,7 @@ mod tests {
 
     #[test]
     fn test_bernoulli() {
-        let data = Bernoulli::new(0.75).sample_vec(1e6 as usize);
+        let data = Bernoulli::new(0.75).sample_n(1e6 as usize);
         for i in &data {
             assert!(*i == 0. || *i == 1.);
         }
