@@ -71,12 +71,12 @@ pub fn erf(x: f64) -> f64 {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::distributions::{Distribution, Exponential, Uniform};
+    use crate::distributions::{Distribution, Distribution1D, Exponential, Uniform};
     use approx_eq::assert_approx_eq;
 
     #[test]
     fn test_logistic() {
-        let d = Exponential::new(5.).sample_vec(100 as usize);
+        let d = Exponential::new(5.).sample_n(100 as usize);
         d.iter().for_each(|x| {
             assert_approx_eq!(logistic(*x) + logistic(-*x), 1.);
         });
@@ -94,7 +94,7 @@ mod tests {
 
     #[test]
     fn test_logit() {
-        let d = Uniform::new(0., 1.).sample_vec(100 as usize);
+        let d = Uniform::new(0., 1.).sample_n(100 as usize);
         d.iter().for_each(|x| {
             assert_approx_eq!(*x, logistic(logit(*x)));
             assert_approx_eq!(*x, logit(logistic(*x)));
