@@ -1,31 +1,28 @@
 //! Various optimization algorithms (eg. Adam, SGD, Levenberg-Marquardt).
 
+use super::DiffFn;
+use crate::linalg::Vector;
 use autodiff::F1;
 
 mod adam;
+// mod lbfgs;
 mod lm;
 mod sgd;
 
 pub trait Optimizer {
+    type Output;
     fn optimize<F>(
         &self,
-        xs: &[f64],
-        ys: &[f64],
         f: F,
         parameters: &[f64],
+        data: &[&[f64]],
         maxsteps: usize,
-    ) -> Vec<f64>
+    ) -> Self::Output
     where
-        F: Fn(&[F1]) -> F1 + Copy;
-    fn grad_fn_type(&self) -> GradFn;
-}
-
-#[derive(Debug, Clone, Copy)]
-pub enum GradFn {
-    Predictive,
-    Residual,
+        F: DiffFn;
 }
 
 pub use self::adam::*;
+// pub use self::lbfgs::*;
 pub use self::lm::*;
 pub use self::sgd::*;
